#!/usr/bin/perl

# WEBFIX

# Recherche de nologismes par suffixe sur le Web, via ALTAVISTA

# Copyright Ludovic Tanguy - ERSS (C) 2001 
# Ludovic.Tanguy@univ-tlse2.fr

#  Ce programme est un logiciel libre ; vous pouvez le redistribuer
#  et/ou le modifier conformment aux dispositions de la Licence
#  Publique Gnrale GNU, telle que publie par la Free Software
#  Foundation ; version 2 de la licence, ou encore toute version
#  ultrieure.

#  Ce programme est distribu dans l'espoir qu'il sera utile, mais
#  SANS AUCUNE GARANTIE ; sans mme la garantie implicite de
#  COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus
#  de dtail, voir la Licence Publique Gnrale GNU .

#  Vous devez avoir reu un exemplaire de la Licence Publique Gnrale
#  GNU en mme temps que ce programme ; si ce n'est pas le cas,
#  crivez  la Free Software Foundation Inc., 675 Mass Ave,
#  Cambridge, MA 02139, Etats-Unis.


use Getopt::Long;
use locale;
use LWP::Simple;
use URI::URL;

$|=1;

&GetOptions("m"=>\$MAJU, "majuscule" =>\$MAJU, 
	    "a" =>\$ACCENTS, "accents" =>\$ACCENTS,
	    "lang=s" =>\$LANG, "l=s" =>\$LANG,
	    "dico=s" => \$DICOFILE, "d=s" => \$DICOFILE,
	    "prefixe=s" => \$PREFIXFILE, "p=s" => \$PREFIXFILE,
	    "h" => \$HELP, "help" => \$HELP,  "aide" => \$HELP);

$DICOFILE= "dico-francais" unless $DICOFILE;
$LANG="fr" unless $LANG;
$PREFIXFILE= "triplets-francais" unless $PREFIXFILE;

if ($HELP) {
    die "WEBFIX : recherche de suffixes sur le WWW, en interrogeant ALTAVISTA\n
Usage : $0 [-mah] [-lang|l langage] [-dico|d dictionnaire] [-prefixe|p fichier_prfixes] suffixe \n\n
-m, --majuscule \t Recherche galement les mots en majuscules\n
-a, --accents \t\t Prend en compte les accents dans les prfixes et le dictionnaire\n
-l, --lang=XX \t\t Choix du langage pour interroger ALTAVISTA (dfaut : fr)
\t\t\t (fr=franais, it=italien, en=anglais)
\t\t\t voir www.altavista.com pour plus de dtails\n
-d, --dico=fichier \t Liste de mots attests (dfaut : dico-francais)\n\n
-p, --prefixe=fichier \t Liste des triplets de lettres possibles  l'initiale des mots 
\t\t\t (dfaut : triplets-francais)\n
-h, --help \t\t Obtenir ce texte\n\n" };


die "Usage : $0 [-mah] [-lang|l langage] [-dico|d dictionnaire] [-prefixe|p fichier_prfixes] suffixe \nTapez $0 -h pour l'aide\n" unless $#ARGV==0;
$suffixe=$ARGV[0];
$suffixesansaccents=$suffixe;

print STDERR "Pas de prise en compte des accents\n" if $ACCENTS;
print STDERR "Prise en compte des accents\n" unless $ACCENTS;
print STDERR "Filtrage des mots en \"$suffixe\" du dictionnaire $DICOFILE\n";

open(IN, "<$DICOFILE") or die "Fichier introuvable : $DICOFILE ($!)\n";
while(<IN>){
    unless ($_ =~/ /){
	chomp;
	$sansaccents =$_;
	$sansaccents =~tr//aaaeeeeiioouuuc/;
	if (m/$suffixe$/oi || ($ACCENTS && $sansaccents =~m/$suffixesansaccents$/oi )){
	    $pref=substr($_, 0, 3);
	    $_=~tr//aaaeeeeiioouuuc/ if $ACCENTS;
	    $frequent{$pref}.=" -$_" unless $frequent{$pref}=~/\-$_\b/;
	}
    }
}

#  foreach (sort keys %frequent){
#      print  $_,"\t",$frequent{$_},"\n";}

print STDERR "Chargement de la liste des prfixes\n";

open (IN, "<$PREFIXFILE") or die "Fichier introuvable : $PREFIXFILE ($!)\n";
while(<IN>){
    chomp;
    push @MotsCles, $_ unless ($_ =~/^[\#\>]/);}

close IN;

print STDERR "Dbut de la recherche\n";
print STDERR "Pas de recherche des mots en majuscules\n" unless $MAJU;
print STDERR "Le recherche prend en compte les mots en majuscules\n" if $MAJU;

foreach $MotCle(@MotsCles){ 
    print STDERR "$MotCle\n";
    &LanceAV($MotCle);
}

sub LanceAV {
    my $MotCle = $_[0];
    my $url = url('http://www.altavista.com/cgi-bin/query');
    my $result="";
    $url->query_form(
		     q=>"$MotCle*****$suffixe ".$frequent{$MotCle},
		     kl=>$LANG,
		     pg=>'q',
		     wt=>'y',
		     Translate=>'on',
		     );

    if (defined (my $contenu = get $url)) {
#	print $contenu;
	if ($contenu =~/We found (about )?(.*?) results?/i){
	    $result=$2;
	    $result=~s/[,\. ]//g;
	}
	elsif ($contenu =~/AltaVista found no document matching your query/i)
	{$result=0;}
	else {warn "Problme de la part d'AltaVista !\n";}
	if ($result>0) {
	    print "PREFIXE : $MotCle  ($result)\n";
	    print "ATTESTATIONS : ",$frequent{$MotCle},"\n" if $frequent{$MotCle};
	   	    
	    if ($contenu =~/<a href=\"(.*?)\" target=\"_self\">\[Next \&gt\;\&gt\;\]\<\/a\>/){
		$secondpage = "http://www.altavista.com$1";
	    }
	    else {$secondpage="";}
	    
	    while ($contenu =~/r=(http:\/\/.*?)\"/){
		$contenu=$';
		$site=$1;
		if (defined (my $page = get $1)){
		    if ($MAJU){
		    while ($page =~/\b($MotCle(\w|\-)*$suffixe)/ig){
			print "$1\t$site\n" unless $frequent{$MotCle}=~/\-$1/i;}
		}
		else {
		    while ($page =~/\b($MotCle(\w|\-)*$suffixe)/g){
			print "$1\t$site\n" unless $frequent{$MotCle}=~/\-$1/i;}
		}
		
	    }
	    }	    
	    if ($secondpage){
		if (defined (my $contenu = get $secondpage)) {
		    
		    while ($contenu =~/r=(http:\/\/.*?)\"/){
			$contenu=$';
			$site=$1;
			if (defined (my $page = get $1)){
			    if ($MAJU){
				while ($page =~/\b($MotCle(\w|\-)*$suffixe)/ig){
				    print "$1\t$site\n" unless $frequent{$MotCle}=~/\-$1/i;}
			    }
			    else {
				while ($page =~/\b($MotCle(\w|\-)*$suffixe)/g){
				    print "$1\t$site\n" unless $frequent{$MotCle}=~/\-$1/i;}
			    }
			}
		    }
		}		
	    }
	}
    }
}
