# xTree.pl
# t0by
# History:
# 1.00 - Created.  xTree and error stuff.
# 1.01 - Added ole/com event support.
#        We can now catch the status of the doc and do async stuff also.
# 1.02 - Added validation support.  validateOnParse
$VERSION = '1.01';
use Win32::OLE;
use Win32::OLE qw(EVENTS);
use overload;
#/-------------------------/ check args
my $xmlfile = "";
my $validateOnParse = 0;
if (!@ARGV) { &usage; }
else { 
	$xmlfile = $ARGV[0];
	if (defined($ARGV[1])) {
		$validateOnParse = 1;
	}
}
#/-------------------------/ setup 
#Indicates state of the xml document
$STATE_LOADING     = 1;
$STATE_LOADED      = 2;
$STATE_INTERACTIVE = 3;
$STATE_COMPLETED   = 4;
#/-------------------------/ create object 
#$xDoc = new Win32::OLE("MSXML2.DOMDocument.3.0");
$xDoc = new Win32::OLE("MSXML.DOMDocument");
if (&ole_error) { &show_ole_error(__LINE__); }
#Enable ole/com events
Win32::OLE->WithEvents($xDoc, \&event);
#/-------------------------/ main
#Set validation
$xDoc->{validateOnParse} = $validateOnParse;
$xDoc->load($xmlfile);
if (&parse_error) { &show_parse_error(__LINE__); }
print "\nHere is the xTree:\n";
&xTree($xDoc->childNodes, "  ");
undef $xDoc;
#/-------------------------/ subroutines
sub event(){
	if (&ole_error) { &show_ole_error(__LINE__); }
	my ($obj,$evt,@args) = @_;
	SWITCH: {
		$evt =~ m/onreadystatechange/i && do {
			&_onreadystatechanged();
			last SWITCH;
		};
		$evt =~ m/ondataavailable/i && do {
			&_ondataavailable();
			last SWITCH;
		};
		$evt !~ m/[onreadystatechanged|ondataavailable]/i && do {
			&_unknown($evt);
			last SWITCH;
		};
	}
}
sub _onreadystatechanged {
	if (&ole_error) { &show_ole_error(__LINE__); }
	my $state = $xDoc->readyState;
	SWITCH: {
		$state == $STATE_LOADING && do {
			print "Loading ...\n";
			last SWITCH;
		};
		$state == $STATE_LOADED && do {
			print "Loaded ...\n";
			last SWITCH;
		};
		$state == $STATE_INTERACTIVE && do {
			print "Interactive ...\n";
			last SWITCH;
		};
		$state == $STATE_COMPLETED && do {
			print "Completed ...\n";
			last SWITCH;
		};
	}
}
sub _ondataavailable {
	if (&ole_error) { &show_ole_error(__LINE__); }
	print "Data available ...\n";
}
sub _unknown() {
	if (&ole_error) { &show_ole_error(__LINE__); }
	my $evt = shift;
	print "\n**Unknown event fired ...\n", $evt, "\n";
	exit;
}	

sub xTree() {
	my ($nodetree,$indent) = @_;
	my ($nodelen, $attrlen, $nodelist, $node, $attr);
	$indent = $indent . "  ";
   	foreach my $nodelist ($nodetree) {
   		my $nodelen = $nodelist->length;
   		for (my $i=0; $i<$nodelen; $i++) {
   			my $node = $nodelist->item($i);
				print $indent . $node->nodeName, ":", $node->nodeValue, "\n";
				#check for attribures
				if ($node->attributes > 0) {
					$attrlen = $node->attributes->length;
					for (my $j=0; $j<$attrlen; $j++) {
						$attr = $node->attributes->item($j);
						print $indent . "@" . $attr->nodeName, ":", $attr->nodeValue, "\n";
					}
				}
				#check for children
				if ($node->childNodes->length > 0) {
   				&xTree($node->childNodes, $indent);
   			}
			}
   	}
}

sub parse_error {
	if ($xDoc->parseError->errorCode != 0) {
		return 1;
	}
	return undef;
}

sub show_parse_error() {
	my $line = shift;
	my $err = "\n... PARSE ERROR ...\n\n";
	$err .= " XML File: "   . $xDoc->parseError->url       . "\n";
	$err .= " Error Code: " . $xDoc->parseError->errorCode . "\n";
	$err .= " Reason: "     . $xDoc->parseError->reason;
	$err .= " Line: "       . $xDoc->parseError->line      . "\n";
	$err .= " Text: "       . $xDoc->parseError->srcText   . "\n";
	print $err;
	exit;
}

sub ole_error {
	if (Win32::OLE->LastError != 0) {
		return 1;
	}
	return undef;
}

sub show_ole_error() {
	my $line = shift; $line--;
	print "\n**Error on line $line of $0 ...\n", Win32::OLE->LastError(), "\n";
	exit;
}



sub usage {
	print "Usage: perl $0 xml-file [validate]\n";
	exit;
}

#/-------------------------/ eof
