package Tk::TextTools;


use strict;
use Tk qw( 800.022 Ev Exists );

use Storable qw( freeze thaw );
use DB_File;

use AutoLoader;

use base qw( Tk::TextEdit );
Construct Tk::Widget 'TextTools';
use locale;

use vars( qw/$VERSION $top1 $top2 $top3 @a $BTREE %b $FORMS %f $DIST %d/ );
$VERSION = '2.00';

sub ClassInit {
    
    my ( $class, $parent ) = @_;
    $class->SUPER::ClassInit( $parent );

    $parent->bind( $class, '<Control-n>', 'ConfirmEmptyDocument' );
    $parent->bind( $class, '<Control-s>', 'FileSave' );
    $parent->bind( $class, '<Control-e>', 'FileExportPopup' );
    $parent->bind( $class, '<Control-f>', 'FindPopUp' );
    $parent->bind( $class, '<Control-r>', 'FindAndReplacePopUp' );
    $parent->bind( $class, '<F6>',        'FormalProximity' );

    return( $class );
}


sub InitObject {
    
    my ( $self ) = @_;
    $self->SUPER::InitObject;

    $self->tagConfigure( 'match', -foreground=>'white', -background =>'black');
    $self->tagConfigure( 'CURSOR_HIGHLIGHT_TAGBRACES',-background=>"#828ee062ffff" );

    $self->tagConfigure( 'BLOCK_HIGHLIGHT_TAGBRACES',-background => 'red' );
    $self->tagConfigure( 'BLOCK_HIGHLIGHT_CURLIES',  -background => 'red' );

    $self->tagRaise('BLOCK_HIGHLIGHT_TAGBRACES', 'CURSOR_HIGHLIGHT_TAGBRACES');
    %{$self->{'HIGHLIGHT_PAIR_DESCRIPTOR_HASH_REF'}}->{'TAGBRACES'}
    = [ 'multiline', '<', '>' ];

    return( $self->_setMatchingcouples );
}


sub _setMatchingcouples {
    
    my ( $self ) = @_;
    my ( $value, $i, $direction );

    $value = "[]{}()<>";
    for ( $i = 0; $i < length( $value ); $i++ ) {
	$direction = ( $i % 2 ? -1 : 1 );
	if (( $direction == -1 ) && ( substr( $value, $i, 1 ) eq
				      substr( $value, $i+$direction, 1 ))) {
	    next();
	}
	$self->{MATCHINGCOUPLES}->{substr( $value, $i, 1)}
	= [substr( $value, $i+$direction, 1 ), $direction];
    }
    return( $self );
}


sub _FindMatchingChar {

    my ( $self, $search, $pos, $slimit, $elimit ) = @_;
    my ( $pair, $direction, $spos, $d, $p, $c, $match );

    $pair = ${$self->{MATCHINGCOUPLES}->{$search}}[0];
    if (! defined( $pair )) { return( undef ); }
	
    $direction = ${$self->{MATCHINGCOUPLES}->{$search}}[1];
    $spos = ( $direction == 1 ? $self->index( "$pos + $direction c" ) :
              $self->index( $pos ));
    $d = 1;
    if ( $direction == 1 ) {
        $match = "[\\$pair|\\$search]+";
        for ( $p = $spos; $self->compare( $p, '<', $elimit);
	      $p = $self->index( "$p + 1c" )) {

	    $p = $self->search( '-forwards','-regexp','--',$match,$p,$elimit );
	    if (! defined( $p )) { return( undef ); }
	    $c = $self->get( $p );
	    if ( $c eq $pair ) {
		$d--;
		if( $d == 0 ) {
		    return( $p );
		}
	    }
	    elsif( $c eq $search ) {
		$d++;
	    }
	    Tk::DoOneEvent( Tk::DONT_WAIT );
	}
    }
    else {
        $match = "[\\$search|\\$pair]+";
        for ( $p = $spos; $self->compare( $p, '>=', $slimit); ) {
	    $p = $self->search('-backwards','-regexp','--',$match,$p,$slimit );
	    if (! defined( $p )) { return( undef ); }
	    $c=$self->get($p);
	    if ( $c eq $pair ) {
		$d--;
		if( $d == 0 ) {
		    return( $p );
		}
	    }
	    elsif ( $c eq $search ) {
		$d++;
	    }
	    if ( $self->compare( $p, '==', '1.0' )) { return( undef );}
	    Tk::DoOneEvent(Tk::DONT_WAIT);
	}
    }
    return( undef );
}


sub InsertKeypress {

    my ( $self, $character ) = @_;
    my ( $s );

    $s = $self->index( 'insert' );
    if ( $self->OverstrikeMode ) {
	my ( $current );

	$current=$self->get( 'insert' );
	$self->delete( 'insert' ) unless ( $current eq "\n" );
    }
    $self->Insert( $character );

    if (! defined( $self->tagRanges( 'sel' ))) {
	if( exists( %{$self->{MATCHINGCOUPLES}}->{$character} )) {

	    my ( $lines, $colon, $slimit, $elimit, $i );
	    
	    ( $lines, $colon )   = split( '\.', $self->index( 'end' ));
	    ( $slimit, $elimit ) = $self->yview;
	    
	    $slimit = int( $lines * $slimit ) + 1;
	    $slimit = "$slimit.0";
	    $elimit = int( $lines * $elimit );
	    $elimit = "$elimit.0";

	    $i = $self->_FindMatchingChar( $character, $s, $slimit, $elimit );
	    if ( defined $i ) {
		my ( $sel );

		$sel = Tk::catch {
		    $self->tag( 'nextrange', 'match', '1.0', 'end' );
		};
		if( defined( $sel )) {
		    $self->tag( 'remove', 'match', 'match.first' );
		}
		$self->tag( 'add', 'match', $i, $self->index( "$i + 1 c" ));
		$self->after( 1400, [\&removeMatch, $self, $i] );
	    }
	}
    }
    return( $self );
}


sub removeMatch {
    
    my ( $self, $i ) = @_;
    
    if ( defined( $i )) { $self->tag( 'remove', 'match', $i ); }
    else { $self->tag( 'remove', 'match', '1.0', 'end' ); }
    return( $self );
}


sub HighlightTagBracesAroundCursor {
    
    my ( $self ) = @_;
    return( $self->HighlightSinglePairBracketingCursor( '<', '>', '[<>]',
    'CURSOR_HIGHLIGHT_TAGBRACES', 'BLOCK_HIGHLIGHT_TAGBRACES', 0 ));
}


sub HighlightAllPairsBracketingCursor {
    
    my ( $self ) = @_;
    
    $self->HighlightParenthesisAroundCursor;
    $self->HighlightCurlyBracesAroundCursor;
    $self->HighlightBracesAroundCursor;
    $self->HighlightDoubleQuotesAroundCursor;
    $self->HighlightTagBracesAroundCursor;
    return( $self );
}


sub menu {
    
    my ( $self ) = @_;
    return( $self->SUPER::menu() );
}


sub MenuLabels {

    my ( $self ) = @_;
    return( qw[~File ~Edit ~Search ~Tools ~View] );
}


sub FileTypes {

    my ( $self ) = @_;
    return( [["Text Files", '.txt'], ["All Files", '*']] );
}


sub loadStatusbar {
    
    my ( $self, $statusbar ) = @_;
    
    $self->{STATUS_MESSAGE} = "Ready for edits!";
    return( $statusbar->configure( -textvariable => \$self->{STATUS_MESSAGE}));
}		   


sub tellStatusbar {
    
    my ( $self, $message ) = @_;
    return( $self->{STATUS_MESSAGE} = $message );
}


sub FileMenuItems {

    my ( $self ) = @_;

    return( [['command' => '~New', -command => [$self,'ConfirmEmptyDocument'],
	      -accelerator => 'Ctrl+N'],
	     ['command' => '~Open',    -command => [$self, 'FileLoadPopup']],
	     ['command' => '~Include', -command => [$self,'IncludeFilePopup']],
	     '_',
	     ['command' => '~Save',    -command => [$self, 'FileSave' ],
	      -accelerator => 'Ctrl+S' ],
	     ['command' => 'Save ~As', -command => [$self, 'FileSaveAsPopup']],
	     ['command' => 'Export ~To', -command=> [$self, 'FileExportPopup'],
	      -accelerator => 'Ctrl+E'],
	     '-',
	     ['command' => '~Exit',    -command => [$self, 'ConfirmExit']]] );
}


sub EditMenuItems {

    my ( $self ) = @_;

    return( [['command' => '~Undo',   -command => [$self, 'undo'],
	      -accelerator => 'Ctrl+_'],
	     ['command' => '~Redo',   -command => [$self, 'redo']],
	     '-',
	     ['command' => '~Cut',   -command => [$self, 'clipboardCut'],
	      -accelerator => 'Ctrl+X'],
	     ['command' => 'C~opy',  -command => [$self, 'clipboardCopy'],
	      -accelerator => 'Ctrl+W'],
	     ['command' => '~Paste', -command => [$self, 'clipboardPaste'],
	      -accelerator => 'Ctrl+Y'],
	     ['command' =>'C~lear',  -command => [$self, 'deleteSelected']],
	     '-',
	     ['command' =>'Select ~All', -command => [$self, 'selectAll'],
	      -accelerator => 'Ctrl+/'],
	     ['command' => 'U~nselect All',
	      -command  => [$self, 'unselectAll']]] );
}


sub SearchMenuItems {

    my ( $self ) = @_;

    return( [['command' => '~Find', -command => [$self, 'FindPopUp'],
	      -accelerator => 'Ctrl+F'],
	     ['command' => 'Find ~Next',
	      -command  => [$self, 'FindSelectionNext']],
	     ['command' => 'Find ~Previous',
	      -command  => [$self, 'FindSelectionPrevious']],
	     '-',
	     ['command' =>'~Replace',
	      -command  => [$self, 'FindAndReplacePopUp'],
	      -accelerator => 'Ctrl+R']] );
}


sub ToolsMenuItems {

    my ( $self ) = @_;

    return( [['command' => 'Formal ~Proximity',
	      -command  => [$self, 'FormalProximity'],
	      -accelerator => 'F6']] );
}


sub ConfirmEmptyDocument {
    
    my ( $self ) = @_;
    
    if ( $self->ConfirmDiscard ) {
	
	$self->tellStatusbar( "Discarding buffer..." );
	$self->EmptyDocument();
	return( $self->tellStatusbar( "New file loaded." ));
    }
    return( 0 );
}


sub FileLoadPopup {

    my ( $self ) = @_;
    if ( $self->ConfirmDiscard ) {

	my ( $file ) = $self->CreateFileSelect( 'getOpenFile',
						-title =>"File Open",
						-filetypes=> $self->FileTypes);
	if ( defined( $file ) and length( $file )) {
	    
	    $self->tellStatusbar( "Opening \"$file\"..." );
	    $self->Load( $file );
	    return( $self->tellStatusbar( "File \"$file\" opened." ));
	}
	else {
	    return( $self->tellStatusbar( "Action chanceled!" ));
	}
    }
    return( 0 );
}


sub IncludeFilePopup {

    my ( $self ) = @_;
    my ( $file ) = $self->CreateFileSelect( 'getOpenFile',
					    -title => "File Include",
					    -filetypes => $self->FileTypes );
    if ( defined ( $file ) and length( $file )) {
	
	$self->tellStatusbar( "Including \"$file\"..." );
	$self->IncludeFile( $file );
	return( $self->tellStatusbar( "File \"$file\" included." ));
    }
    else {
	return( $self->tellStatusbar( "Action chanceled!" ));
    }

    return( 0 );
}


sub FileSave {

    my ( $self ) = @_ ;
    my ( $file ) = $self->FileName();

    if ( defined( $file ) and length( $file )) {
	
	$self->tellStatusbar( "Saving \"$file\"..." );
	$self->Save( $file );
	return( $self->tellStatusbar( "File \"$file\" saved." ));
    }
    else {
	return( $self->FileSaveAsPopup );
    }
    return( 0 );
}


sub FileSaveAsPopup {
    
    my ( $self ) = @_;
    my ( $file ) = $self->CreateFileSelect( 'getSaveFile',
					    -title => "File Save As",
					    -filetypes => $self->FileTypes );
    if ( defined( $file ) and length( $file )) {

	$self->tellStatusbar( "Saving file as \"$file\"..." );
	$self->Save( $file );
	return( $self->tellStatusbar( "File saved as \"$file\"." ));
    }
    else {
	return( $self->tellStatusbar( "Action chanceled!" ));
    }
    return( 0 );
}


sub FileExportPopup {

    my ( $self ) = @_;
    my ( $selection );

    eval { $selection = $self->SelectionGet( -selection=>"CLIPBOARD" );};
    unless ( $@ ) {
	if ( defined( $selection ) and length( $selection )) {
	    
	    my ( $file ) = $self->CreateFileSelect( 'getSaveFile',
					       -title => "Export To File",
					       -filetypes=> $self->FileTypes );
	    if ( defined( $file ) and length( $file )) {
		
		$self->tellStatusbar( "Exporting clipboard to \"$file\"..." );
		open ( FILE, ">>$file" ) or die "Could not open $file! $!\n" ;
		print FILE "$selection\n\n";
		close( FILE );
		return( $self->tellStatusbar
			("Clipboard exported to \"$file\"." ));
	    }
	    else {
		return( $self->tellStatusbar( "Action chanceled!" ));
	    }
	}
    }
    return( 0 );
}


sub ConfirmDiscard {

    my ( $self ) = @_;
    my ( $question, $answer );

    if ( $self->numberChanges ) {
	$question = "Do you want to save edits ?";
	$answer = $self->messageBox( -icon    => 'question',
				     -title   => 'Question',
				     -type    => 'YesNoCancel',
				     -message =>  $question );
	
	if ( $answer eq 'yes' ) {
	    return( $self->Save );
	}
	elsif( $answer eq 'cancel' ) {
	    return( 0 );
	}
	else {
	    $question = "The buffer has been modified.\nClose it anyway ?";
	    $answer   = $self->messageBox( -icon    => 'warning',
					   -title   => 'Warning',
					   -type    => 'OKCancel',
					   -message => $question ); 
  
	    if ( $answer eq 'ok' ) {
		return( 1 );
	    }
	    return( 0 );
	}
    }
    return( 1 );
}


sub ConfirmExit {

    my ( $self ) = @_;

    if ( $self->ConfirmDiscard ) {
	$self->tellStatusbar( "Leaving application..." );
	return( $self->toplevel->destroy() );
    }
    return( 0 );
}


sub FindNext {

    my ( $self, $direction, $mode, $case, $pattern ) = @_;
    my ( $start_index, $return );

    $self->cancel_current_gui_callback_and_restart_from_beginning;

    eval {
	if ( $direction eq '-forward' ) {
	    $self->markSet( 'insert', 'sel.last' );
	    $self->markSet( 'current', 'sel.last' );
	}
	else {
	    $self->markSet( 'insert', 'sel.first' );
	    $self->markSet( 'current', 'sel.first' );
	}
    };

    my ( $saved_index ) = $self->index('insert');
    $self->unselectAll;
    my ( $match_length ) = 0;
	
    if ( $case eq '-nocase' ) {

	$start_index = $self->search
	    ( $direction,
	      $mode,
	      $case,
	      -count => \$match_length,
	      "--",
	      $pattern ,
	      'insert',
	      $direction eq '-forward'? 'end': '1.0' );
    }
    else {

	$start_index = $self->search
	    ( $direction,
	      $mode,
	      -count => \$match_length,
	      "--",
	      $pattern ,
	      'insert',
	      $direction eq '-forward'? 'end': '1.0' );
    }
	
    unless( defined( $start_index )) { return( 0 ); }
    if ( length( $start_index ) == 0 ) { return( 0 ); }
	
    my ( $line, $column ) = split( /\./, $start_index );
    $column = $column + $match_length;
    my ( $end_index ) = $line.'.'.$column;

    $self->tagAdd( 'sel', $start_index, $end_index );
    $self->see( $start_index );

    if ( $direction eq '-forward' ) {
	
	$self->markSet( 'insert', $end_index );
	$self->markSet( 'current', $end_index );
    }

    else {
	
	$self->markSet( 'insert', $start_index );
	$self->markSet( 'current', $start_index );
    }

    my ( $compared_index ) = $self->index( 'insert' );
    if ( $compared_index eq $saved_index ) {
	$return = 0;
    }

    else {
	$return = 1;
    }

    return( $return );
}


sub FindAll {

    my ( $self, $mode, $case, $pattern ) = @_;
    my ( $start_index );

    $self->cancel_current_gui_callback_and_restart_from_beginning;
    $self->unselectAll;
    
    my ( $match_length ) = 0;
    my ( $end_index ) = '1.0';	
    
    while ( defined( $end_index )) {

	if ($case eq '-nocase') {
	    $start_index = $self->search
		( $mode,
		  $case,
		  -count=>\$match_length,
		  "--",
		  $pattern,
		  $end_index,
		  'end' );
	}
	else {
	    $start_index = $self->search
		( $mode,
		  -count => \$match_length,
		  "--",
		  $pattern,
		  $end_index,
		  'end' );
	}

	unless ( defined( $start_index ) && $start_index ) { last; }

	my ( $line, $colon ) = split( /\./, $start_index );
	$colon = $colon + $match_length;
	$end_index = $line.'.'.$colon;

	$self->tagAdd( 'sel', $start_index, $end_index );
    }
    return( $self );
}


sub selectAll {

    my ( $self ) = @_;

    $self->cancel_current_gui_callback_and_restart_from_beginning;
    return( $self->tagAdd( 'sel', '1.0', 'end' ));
}


sub ReplaceSelectionsWith {

    my ( $self, $new_text ) = @_;

    my ( @ranges ) = $self->tagRanges( 'sel' );
    if ( @ranges ) {
	my ( $i );

	for ( $i=0; $i<=$#ranges; $i++ ) {

	    $self->markSet( 'mark_sel_'.$i => $ranges[$i] );
	}
	for ( $i=0; $i<=$#ranges; $i=$i+2 ) {

	    my ( $first ) = $self->index( 'mark_sel_'.$i );
	    my ( $last ) = $self->index( 'mark_sel_'.( $i+1 ));   
	    $self->insert( $last, $new_text );
	    $self->delete( $first, $last );
	}
	$self->markSet( 'insert', $self->index( 'mark_sel_'.$#ranges ));

	for ( $i=0; $i<=$#ranges; $i++ ) {
	    $self->markUnset( 'mark_sel_'.$i ); 
	}
    }
    return( $self );
}


sub FindAndReplaceAll {

    my ( $self, $code1, $code2, $mode, $case,
	 $pattern1, $pattern2, $atts ) = @_;
    my ( $progress, $count );

    $self->tellStatusbar( "Starting global edits..." );

    $self->MainWindow->Busy;
    $self->addGlobStart;

    my ( $end ) = $self->index( 'end' );
    $self->markSet( 'insert', '1.0' );
    $self->unselectAll;

    while ( $self->$code1( '-forward', $mode, $case, $pattern1 )) {
	
	my ( $line ) = $self->index( 'sel.first' );
	my ( $index_line ) = $line;
	$index_line =~ s /\.\d+$//;

	if (( $count++%100 ) == 0 ) {
	    $progress = $self->TextUndoFileProgress(
		        "Global edits" => $self->FileName(),
		        $index_line, $line, $end -1 );
	    
	}
	$self->$code2( $pattern2, $atts );
    }

    if ( defined ( $progress )) {
	$progress->withdraw();
    }

    $self->addGlobEnd;
    $self->MainWindow->Unbusy;

    return( $self->tellStatusbar( "End of the global edits." ));
}


sub FindPopUp {
    
    my ( $self ) = @_;
    return( $self->findandreplacepopup( "Find String" ));
}


sub FindAndReplacePopUp {

    my ( $self ) = @_;
    return( $self->findandreplacepopup( "Find And Replace" ));
}


sub makeRegex {

    my ( $self, @list ) = @_;
    my ( %hash );

    foreach ( @list ) {
	next if ( /^\s*$/ );
	$hash{$_}++;
    }
    @list = sort( keys( %hash ));
    return( $self->CommonPrefix( "", @list ));
}


sub CommonPrefix {

    my ( $self, $p, @list ) = @_;
    my ( %hash, $prefix, @all, $paren );

    return( "$p@list" ) if ( @list == 1 );
  
    foreach ( sort( @list )) {
	my ( $suffixed_word );

	$prefix = substr( $_, 0, 1 );
	$suffixed_word = length( $_ )? substr( $_, 1 ): "";
	push ( @{$hash{$prefix}}, $suffixed_word );
    }
  
    foreach ( sort( keys( %hash ))) {
	my ( $comm );

	$comm = $self->CommonPrefix( $_, @{$hash{$_}} );
	$comm = " " if ( $comm eq "" );   
	push( @all, $comm );
    }

    $paren="";
    if ( @all == 1 ) {
	$paren = "@all";
    }
    else {
	my ( $len, $mark, $join_char, $join );

	$len = 0;  
	foreach ( @all ) {
	    my ( $l );

	    $l = length( $_ );
	    $len = $l > $len ? $l : $len;
	}
	
	$mark = ( grep /^ /, @all ) ? "?" : "";        
	$join_char = ( $len == 1 ) ? "" : "|";
	$join = join "$join_char", grep !/ /, @all;

	if ( length( $join ) == 1 ) {
	    $paren = "$join$mark";    
	}
	else {
	    $paren = $len == 1 ? "[$join]$mark" : "($join)$mark";    
	}  
    }
    return( "$p$paren" );
}


sub findandreplacepopup {

    my ( $self, $action )= @_;
    if (! Exists( $top1 )) {

	my ( $pattern1, $pattern2, $checked1, $checked2, $checked3,
	     $frame1, $frame2, $frame3, $frame4, @ranges, $selection,
	     $first, $last, $button_ok, $button_next,
	     $button_all, $button_cancel, $direction, $mode, $case );

	$self->tellStatusbar( "Opening assisted session..." );
	$top1 = $self->Toplevel( -title => $action );
	

	$frame1 = $top1->Frame()->grid( -row => 0, -column => 0, -ipadx => 10,
		   -ipady => 5, -sticky => 'ew' ); 
	$frame1->Label( -text => "    Find What: " )->grid
	              ( -row => 0, -column => 0 );
	my ( $entry1 ) = $frame1->Entry( -textvariable => \$pattern1,
		       -width => 35, -selectforeground => 'white',
		       -selectbackground => "#000070669645")->grid( -row => 0,
					  -column => 1, -ipady => 5 );
	$entry1->bind( '<Button-3>', sub {

	    eval { $selection= $self->SelectionGet( -selection=> "PRIMARY" );};
	    unless( $@ ) {
		if ( defined( $selection ) and length( $selection )) {
		    $entry1->insert( 'end', $self->makeRegex
				     ( split( /\s+/, $selection )));
		}
	    }
	});

	if ( $action eq "Find And Replace" ) {
	    
	    $frame2 = $top1->Frame()->grid( -row => 1, -column => 0,
		      -ipadx => 10, -ipady => 5, -sticky => 'ew' );
	    $frame2->Label( -text => "Replace With: ")
		->grid( -row => 0, -column => 0 );
	    $frame2->Entry( -textvariable => \$pattern2, -width => 35,
			-selectforeground => 'white',
			-selectbackground => "#000070669645" )->grid
			    ( -row => 0, -column => 1, -ipady => 5 );
	    $pattern2 = "";
	}
	
	$frame3 = $top1->Frame()->grid( -row => 2, -column => 0,
		   -ipadx => 10, -ipady => 5, -sticky => 'w' );
	$frame3->Checkbutton( -text => "Search Backwards",
			      -variable => \$checked1 )->grid
				  ( -row => 0, -column => 0, -sticky => 'w' );
	$frame3->Checkbutton( -text => "Regexp Search",
			      -variable => \$checked2 )->grid
				  ( -row => 0, -column => 1, -sticky => 'w' );
	$frame3->Checkbutton( -text => "Ignore Case",
			      -variable => \$checked3 )->grid
				  (-row => 0,-column => 2, -sticky => 'w' );
    
    
	$frame4 = $top1->Frame( -relief=> 'groove', -borderwidth=> 2 )->grid
	( -row => 3, -column => 0, -ipadx => 10, -ipady=> 10, -sticky=> 'ew' );

	if ( $action eq "Find And Replace" ) {
	    $button_ok = $frame4->Button( -text => "OK", -command => sub {
	    
	    $self->ReplaceSelectionsWith( $pattern2 );
	    $button_next->invoke();
	})->grid( -row => 0, -column => 0,-sticky => 'nsew', -padx => 15 );
	}
	$button_next = $frame4->Button( -text => "Next", -command => sub {
	    
	    $direction = $checked1? '-backward': '-forward';
	    $mode      = $checked2? '-regexp': '-exact';
	    $case      = $checked3? '-nocase': '-case';
	
	    if ( defined( $pattern1 ) and length( $pattern1 )) {
		$self->FindNext( $direction, $mode, $case, $pattern1 );
	    }
	})->grid( -row => 0, -column => 1, -padx => 15, -sticky => 'nsew' );
	$button_all = $frame4->Button( -text => "All", -command => sub {

	    $mode      = $checked2? '-regexp': '-exact';
	    $case      = $checked3? '-nocase': '-case';

	    if ( defined( $pattern1 ) and length( $pattern1 )) {
		
		if ( $action eq "Find String" ) {
		    $self->FindAll( $mode, $case, $pattern1 );
		}
		else {
		    $self->FindAndReplaceAll
			( \&FindNext, \&ReplaceSelectionsWith,
			  $mode, $case, $pattern1, $pattern2 );
		}
	    }
	})->grid( -row => 0, -column => 2, -padx => 15, -sticky => 'nsew');
	$button_cancel = $frame4->Button( -text => "Cancel", -command => sub {
	    $self->tellStatusbar( "Assisted session closed." );
	    $top1->destroy();
	})->grid( -row => 0, -column => 3, -sticky => 'nsew', -padx => 15 );

    
	@ranges = $self->tagRanges( 'sel' );
	if ( @ranges ) {
	
	    my ( $first_line, $first_colon, $last_line, $last_colon );
	
	    $first = shift( @ranges );
	    $last = shift( @ranges );
	    ( $first_line, $first_colon ) = split( /\./, $first );
	    ( $last_line, $last_colon ) = split( /\./, $last );
	
	    unless( $first_line == $last_line ) { $last = $first. ' lineend'; }
	    $pattern1 =  $self->get( $first , $last );
	    $entry1->selectionTo( 'end' );
	    $entry1->focus();
	}
	else {
	
	    eval { $selection= $self->SelectionGet( -selection=> "PRIMARY"); };
	    unless( $@ ) {
		if ( defined( $selection ) and length( $selection )) {
		    $pattern1 =  $selection ;
		    $entry1->selectionTo( 'end' );
		    $entry1->focus();
		}
	    }
	}
	$top1->resizable( 0, 0 );
	return( $self->tellStatusbar( "Assisted session opened." ));
    }
    return( 0 );
}


sub makeIndex {

    my ( $self ) = @_;
    my ( $progress, $count, $keys );
 
    $self->tellStatusbar( "Indexing forms..." );

    my ( $end )  = $self->index( 'end' );
    $self->markSet( 'insert', '1.0' );
    $self->unselectAll;
    $a[0] = "locked";

    while ( $self->FindNext( '-forward', '-regexp', 0,
			     "[^\.\?\!]+[\.\?\!]?" )) {

	my ( $sentence ) = $self->get( 'sel.first', 'sel.last' );
	if ( $sentence =~ /\w/ ) {

	    while ( $sentence =~ s/<[^>]+>//) {
		my ( $partition ) = $&;

		if ( $partition =~ /^<key (.+?)=\"?([0-9]+)\"?\/?>$/ ) {
		    $keys->{$1}->{$2} = $count? $count: 0;
		}
	    }

	    if ( $sentence =~ /\w/ ) {

		my ( $location );
		$location->[0] =  $self->index( 'sel.first' );
		$location->[1] =  $self->index( 'sel.last' );
		
		my ( $line ) = $location->[0];
		$line =~ s /\.\d+$//;

		if (( $count++%25 ) == 0 ) {
		    $progress = $self->TextUndoFileProgress(
				"Indexing forms" => $self->FileName(),
				$line, $location->[0], $end -1 );
		    
		}
	    
		$a[$count] = freeze( $location );
	    
		while ( $sentence =~ s/(\w+)// ) {
		    $b{$1} .= " $count";
		}
	    }
	}
    }

    if ( defined( $progress )) {
	$progress->withdraw();
    }

    $self->sortTable( $BTREE );
    $self->tellStatusbar( "Made index of forms." );

    return( $keys );
}


sub sortTable {

    my ( $self, $REF ) = @_;
    my ( $status, $key, $value, $size );
    
    $self->tellStatusbar( "Sorting the table..." );
    $key = $value = 0;
    
    for ( $status = $REF->seq( $key, $value, R_FIRST );
	  $status == 0 ; $status = $REF->seq( $key, $value, R_NEXT)) {
	my ( $adress );
	
	while ( $value =~ s/(\d+)// ) {
	    $adress->{$1}++;
	}
	eval { $REF->put( $key, freeze( $adress ));
	       if (! $@ ) { $size++; }};
    }

    $REF->put( "_size", $size? $size: 0 );
    return( undef( $REF ));
}


sub searchColocations {

    my ( $self, $entries ) = @_;
    my ( $candidates, $results );

    $self->tellStatusbar( "Searching colocations..." );

    foreach ( @$entries ) {
	if ( $b{$_} ) {
	    my ( $adress ) = thaw( $b{$_} );
	    for ( keys ( %$adress )) {
		$candidates->{$_}++ ;
	    }
	}
	else { last ; }
    }
    
    for ( keys ( %$candidates )) {
	if ( $candidates->{$_} == @$entries ) {
	    $results->{$_} = 1;
	}
    }

    return ( $results );
}


sub filterForms {

    my ( $self, $results ) = @_;
    my ( $status, $key, $value, $size, $progress, $count );

    $self->tellStatusbar( "Filtering forms... " );
    $key = $value = 0;
    
    for ( $status = $BTREE->seq( $key, $value, R_FIRST );
	  $status == 0; $status = $BTREE->seq( $key, $value, R_NEXT)) {

	if ( $key eq "_size" ) {}
	else {
	    
	    if (( $count++%100 ) == 0 ) {
		$progress = $self->TextUndoFileProgress(
			 "Filtering forms" => $self->FileName(),
			 $count, $count, $b{"_size"} );
		
	    }
	    my ( $adress ) = thaw( $b{$key} );
	    my ( $forms );

	    for ( keys( %$adress )) {
		if ( exists( $results->{$_} )) {
		    $forms->{$_} = 1;
		}
	    }
	    if ( $forms ) {
		$f{$key} = freeze( $forms );
		$size++;
	    }
	}
    }

    if ( defined( $progress )) {
	$progress->withdraw();
    }

    return( $f{"_size"} = $size );
}		


sub checkDistances {
    
    my ( $self, $REF ) = @_ ;
    my ( $status, $key, $value, $size,  $distances, $progress, $count );
    
    $self->tellStatusbar( "Checking distances..." );

    $status = $REF->get( "_size", $size );
    $key = $value = 0;

    for ( $status = $REF->seq( $key, $value, R_FIRST );
	  $status == 0; $status = $REF->seq( $key, $value, R_NEXT )) {

	if ( $key eq "_size" ) {}
	else {

	    if (( $count++%25 ) == 0 ) {
		$progress = $self->TextUndoFileProgress(
			   "Checking distances" => $self->FileName(),
			    $count, $count, $size );
		
	    }

	    my ( $adress ) = thaw( $value );
	    my ( $list ) = [ sort { $a <=> $b }( keys( %$adress )) ];
	    my ( $i, $j );
	
	    for ( $i=0; $i<=$#$list; $i++ ) {
		for ( $j = $i+1; $j<=$#$list; $j++ ) {
		    $d{$list->[$i]} .= " {$list->[$j]}";
		}
	    }
	}
    }

    if ( defined( $progress )) {
	$progress->withdraw();
    }

    $self->sortTable( $DIST );
    return( undef( $REF ));
}


sub checkKeys {
    
    my ( $self, $keys, $index ) = @_ ;
    my ( $references, $key ) ;
 
    foreach $key ( keys ( %$keys )) {
	my ( $candidate );

      SECTIONS: for ( keys ( %{$keys->{$key}} )) {
	  if ( $keys->{$key}->{$_} < $index ) {
	      $candidate = $_ ;
	  }
	  else {
	      last SECTIONS ;
	  }
      }
	$references->{$key} = $candidate;
    }
    return( $references );
}


sub sortDistances {

    my ( $self, $keys, $index ) = @_ ;
    my ( $status, $key, $value, $size );

    $self->tellStatusbar( "Sorting distances..." );
    $key = $value = 0;

    for ( $status = $DIST->seq( $key, $value, R_FIRST );
	  $status == 0 ; $status = $DIST->seq( $key, $value, R_NEXT)) {
	if ( $key eq "_size" ) {}
	else {

	    my ( $reference1 ) = $self->checkKeys( $keys, $key ) ;
	    my ( $adress ) = thaw( $value );
	    
	    foreach ( keys( %{$adress})) {
		my ( $reference2 ) = $self->checkKeys( $keys, $_ ) ;
		if ( $reference1->{$index} eq $reference2->{$index} ) {
		    delete( $adress->{$_} );
		}
	    }
	    if ( %$adress ) {
		$d{$key} = freeze( $adress );
		$size++;
	    }
	    else { delete( $d{$key} ); }
	}
    }
    return( $d{"_size"} = $size );
}


sub SharedWords {
    
    my ( $self, $filter ) = @_;
    my ( $results, $status, $key, $value, $progress, $count );
    
    $self->tellStatusbar( "Finding shared words..." );
    $key = $value = 0;

    for ( $status = $DIST->seq( $key, $value, R_FIRST );
	  $status == 0 ; $status = $DIST->seq( $key, $value, R_NEXT)) {

	if ( $key eq "_size" ) {}
	else {
	    if (( $count++%100 ) == 0 ) {
		$progress = $self->TextUndoFileProgress(
			    "Shared words" => $self->FileName(),
			    $count, $count, $d{"_size"} );
		
	    }
	    my ( $adress ) = thaw( $value );
	    
	    for ( keys( %$adress )) {
		if ( $adress->{$_} >= $filter ) {
		    $results->{$key}->{$_} = 1 ;
		}
	    }
	}
    }

    if ( defined( $progress )) {
	$progress->withdraw();
    }

    return( $results );
}


sub readWords {

    my ( $self, $index ) = @_;
    my ( $words );

    my ( $adress ) = thaw( $a[$index] );
    my ( $sentence ) = $self->get( $adress->[0], $adress->[1] );
    
    while ( $sentence =~ s/(\w+)// ) {
	$words->{$1} = 1;
    }

    return ( $words );
}


sub calculateBigrams {

    my ( $self, $word ) = @_ ;
    my ( $bigrams, $i );
    
    for ( $i=0; $i<=length( $word ) -2; $i++ ) {
	$bigrams->{substr( $word, $i, 2 )} = 1;
    }
    return( $bigrams );
}


sub readBigrams {
    
    my ( $self, $words, $bigrams ) = @_;
    my ( $word, $total_bigrams );
    
    foreach $word ( keys( %$words )) {
	my ( $hash ) = $self->calculateBigrams( $word );
	
	$words->{$word} = scalar( keys( %$hash ));
	$total_bigrams += $words->{$word};
	
	for ( keys( %$hash )) {
	    $bigrams->{$_}->{$word} = 1;
	}
    }

    return( $words, $bigrams, $total_bigrams );
}


sub refinedDistances {
    
    my ( $self, $units ) = @_ ;
    my ( $distances, $unit, $adress ) ;
    
    while (( $unit, $adress ) = each( %$units )) {
	my ( $list ) = [ keys( %$adress ) ];
	
	my ( $i, $j );
	for ( $i=0; $i<=$#$list; $i++ ) {
	    for ( $j = $i; $j<=$#$list; $j++ ) {
		$distances->{$list->[$i]}->{$list->[$j]}++ ;
	    }
	}
    }

    return( $distances );
}


sub calculateProximity {

    my ( $self, $distances, $wordsa, $wordsb ) = @_;
    my ( $proximity, $word );

    foreach $word ( keys( %$distances )) {
	for ( keys( %{$distances->{$word}} )) {
	    if ( $wordsb->{$word} && $wordsa->{$_} ) {

		$proximity->{$word}->{$_}->[1] = $_ ;

		$proximity->{$word}->{$_}->[1] = $self->calculateDice(
			                         $distances->{$word}->{$_},
			       $wordsb->{$word} -$distances->{$word}->{$_},
			          $wordsa->{$_} -$distances->{$word}->{$_} );
		$proximity->{$word}->{$_}->[2] = $distances->{$word}->{$_};
	    }
	    elsif ( $wordsb->{$_} && $wordsa->{$word} ) {

		$proximity->{$_}->{$word}->[0] = $word ;
		$proximity->{$_}->{$word}->[1] = $self->calculateDice(
						 $distances->{$word}->{$_},
			          $wordsb->{$_} -$distances->{$word}->{$_},
			       $wordsa->{$word} -$distances->{$word}->{$_} );
		$proximity->{$_}->{$word}->[2] = $distances->{$word}->{$_} ;
	    }
	}
    }
    return ( $proximity );
}


sub calculateDice {

    my ( $self, $shared, $unitsa, $unitsb ) = @_ ;
    my ( $Dice );
    
    if (( $unitsa + $unitsb ) > 0 ) {
	$Dice = $shared *2/( $unitsa + $unitsb ) ;
    }
    else { $Dice = 100; }
    return ( substr( $Dice, 0, 4 ));
}


sub refineDistances {
    
    my ( $self, $indexa, $indexb ) = @_ ;
    my ( $wordsa, $wordsb, $bigrams, $total_bigramsa,
	 $total_bigramsb, $shared, $proximity );
    
    ( $wordsa, $bigrams, $total_bigramsa ) =
	$self->readBigrams( $self->readWords( $indexa ), $bigrams );
    ( $wordsb, $bigrams, $total_bigramsb ) =
	$self->readBigrams( $self->readWords( $indexb ), $bigrams );
    $shared = 0;

    $proximity = $self->calculateProximity
	( $self->refinedDistances( $bigrams ), $wordsa, $wordsb );

    for ( keys( %$proximity )) {
	my ( $values ) =
	    [ sort {
		$proximity->{$_}->{$b}->[1] <=>
		    $proximity->{$_}->{$a}->[1] ||
			$proximity->{$_}->{$b}->[2] <=>
			    $proximity->{$_}->{$a}->[2];
	    } ( keys ( %{$proximity->{$_}} )) ];
	
	my ( $best_match ) = $values->[0];
	my ( $best_score ) = $proximity->{$_}->{$values->[0]}->[1];
	my ( $level1, $level2 );
	
	foreach $level1 ( keys( %$proximity )) {
	    foreach $level2 ( keys( %{$proximity->{$level1}} )) {
		
		if ( $level2 eq $best_match &&
		     $proximity->{$level1}->{$level2}->[1] > $best_score ) {
		    $best_score = $proximity->{$level1}->{$level2}->[1] ;
		}
	    }
	}
      
	while ( @$values &&
		$best_score > $proximity->{$_}->{$values->[0]}->[1] ) {
	    shift( @$values );
	}
	
	if ( @$values ) {
	    $shared += $proximity->{$_}->{$values->[0]}->[2] ;
	}
    }

    return( $shared, $total_bigramsa -$shared, $total_bigramsb -$shared );
}


sub FormalDistances {
    
    my ( $self, $filter, $units, $score ) = @_ ;
    my ( $status, $key, $value, $results, $progress, $count );
    
    $self->tellStatusbar( "Formal distances..." );
    $key = $value = 0;

    for ( $status = $DIST->seq( $key, $value, R_FIRST );
	  $status == 0 ; $status = $DIST->seq( $key, $value, R_NEXT)) {

	if ( $key eq "_size" ) {}
	else {
	    if (( $count++%1 ) == 0 ) {
		$progress = $self->TextUndoFileProgress(
			   "Formal distances" => $self->FileName(),
			    $count, $count, $d{"_size"} );
 
	    }
	    my ( $adress ) = thaw( $value );
	    for ( sort {$a <=> $b} ( keys( %{$adress} ))) {
		if ( $adress->{$_} >= $filter ) {

		    my ( $Dice, $shared, $unitsa, $unitsb );
		    if ( $units eq 'words' ) {
			
			$shared = $adress->{$_};
			$unitsa = scalar( keys( %{$self->readWords( $key )}))
			    -$adress->{$_} ;
			$unitsb = scalar( keys( %{$self->readWords( $_ )}))
			    -$adress->{$_} ;
		    }
		    elsif ( $units eq 'bigrams' ) {
			( $shared, $unitsa, $unitsb ) =
			    $self->refineDistances( $key, $_ );
		    }

		    $Dice = $self->calculateDice( $shared, $unitsa, $unitsb );
		    if ( $Dice >= $score ) {
			$results->{$key}->{$_}="$Dice $shared $unitsa $unitsb";
		    }
		}
	    }
	}
    }

    if ( defined ( $progress )) {
	$progress->withdraw();
    }
    
    return( $results );
}


sub showProximity {

    my ( $self, $results, $entries, $action ) = @_;	
    $self->tellStatusbar( "Showing the results..." );
	
    $top3 = $self->Toplevel( -title => 'Proximity' );
    my ( $window ) = $top3->Scrolled( 'ROText', -scrollbars => 'osoe',
				      -background => "#ffefffffe0c3",
				      -selectforeground => 'white',
				      -selectbackground => "#000070669645",
				      -height=> 22, -width=> 70,
				      -wrap=> 'word' );
    my ( $menu ) = $top3->Menu();
    $window->tagConfigure( 'error', -background => '#800000',
			   -foreground => '#ffffff' );
    $window->tagConfigure( 'source', -background => "#000070669645",
			   -foreground => "white" );
    $window->tagConfigure( 'index', -background => "#fffbcf" );
    $window->tagConfigure( 'red', -foreground => "red" );
    $window->tagConfigure( 'blue', -foreground => "blue" );
    
    my ( $filemenu ) = $menu->Menubutton( -label => 'File', -tearoff  => 0,
					  -underline => 0, -menuitems => [
				   ['command'   => 'Export ~To',
				   -command => sub {

		       my ( $file )= $self->CreateFileSelect('getSaveFile',
				   -title => "Export To File",
				   -filetypes => $self->FileTypes );
		       if ( defined( $file ) and length( $file )) {
                 
        $self->tellStatusbar( "Saving file \"$file\"..." );
	open( FILE, ">>$file" ) or die( "Can't open \"$file\". $!\n" );
	print FILE $window->get( '1.0', 'end' ), "-" x 10, "\n";

	if ( close( FILE )) {
	    return( $self->tellStatusbar( "File \"$file\" saved." ));
	    }   
        } 
		       else {
			   return( $self->tellStatusbar( "Action chanceled!"));
		       }
		       return( 0 ); }],
									  '-',
		       ['command' => '~Exit',
			-command => sub { $top3->destroy(); }]]);

    $menu->Menubutton( -label => 'View', -tearoff => 0, -underline=>0,
		       -menuitems =>[@{$window->ViewMenuItems}] );

    $top3->resizable( 0, 0 );
    $top3->configure( -menu => $menu );
	
    $window->bind( 'Tk::ROText', '<Button-3>', '' );
    $window->pack();
    
    if (! defined( $results )) {
	$filemenu->menu()->entryconfigure( '0', -state => 'disabled' );
	$window->insert( 'end', "ERROR: No match found!\n\n", 'error' );
    }

    else {
	my ( $key );
	
	$filemenu->menu()->entryconfigure( '0', -state => 'normal' );
	foreach $key ( sort {$a <=> $b} ( keys( %$results ))) {
	    
	    my ( $location ) = thaw( $a[$key] );
	    $window->insert( 'end', "\n" );
	    $window->insert( 'end', "[$key]", 'index' );
	    $window->insert( 'end', $self->get( $location->[0],
						$location->[1] ),'source');
	    $window->insert( 'end', "\n\n" );
	    
	    if ( $action eq 'SHARED WORDS' ) {
		for ( sort {$a <=> $b} ( keys( %{$results->{$key}} ))) {
		    $location = thaw( $a[$_] );
		    $window->insert( 'end', "\t" );
		    $window->insert( 'end', "[$_]", 'index' );
		    $window->insert( 'end', $self->get( $location->[0],
							$location->[1])."\n" );
		}
	    }
	    elsif ( $action eq 'FORMAL DISTANCES' ) {
		for ( sort {$a <=> $b} ( keys( %{$results->{$key}} ))) {
		    $location = thaw( $a[$_] );
		    
		    my ( $Dice, $shared, $unitsa, $unitsb )
			= split( " ", $results->{$key}->{$_} );
		    
		    $window->insert( 'end', "\t" );
		    $window->insert( 'end', "[$_]", 'index' );
		    $window->insert( 'end', $Dice, 'red' );
		    $window->insert( 'end', " $shared $unitsa $unitsb ",
				     'blue' );
		    $window->insert( 'end', $self->get( $location->[0],
							$location->[1] )."\n");
		}
	    }
	}
    }
    return( $self->tellStatusbar( "'Proximity' window displayed." ));
}


sub FormalProximity {
    
    my ( $self ) = @_;

    if (! Exists( $top2 )) {
	my ( $pattern1, $pattern2, $entry1, $entry2,
	     $button1, $button2, $button3, $button_cancel,
	     $keys, @ranges, $selection, $first, $last, $lock );

	$self->tellStatusbar( "Preparing Proximity session..." );
	$self->configure( -state => 'disabled' );

	tie @a, "DB_File", undef, O_RDWR|O_CREAT, 0640, $DB_RECNO 
	    or die "Cannot tie the locations database: $!\n" ;
	$BTREE = tie %b, "DB_File", undef, O_RDWR|O_CREAT, 0640, $DB_BTREE
	    or die "Cannot tie the forms database: $!\n";
	$FORMS = tie %f, "DB_File", undef, O_RDWR|O_CREAT, 0640, $DB_BTREE
	    or die "Cannot tie the filtered forms database: $!\n";
	$DIST = tie %d, "DB_File", undef, O_RDWR|O_CREAT, 0640, $DB_BTREE
	    or die "Cannot tie the distances database: $!\n";
	
	$DB_BTREE->{'compare'} = sub {
	    
	    my( $key1, $key2 ) = @_;
	    
	    if (( $key1 =~ /^\d+$/ ) and ( $key2 =~ /^\d+$/ )) {
		$key1 <=> $key2;
	    }
	    else {
		$key1 cmp $key2;
	    }
	};
	$lock = 1;
	
	my ( $radio ) = "COLOCATIONS";
	my ( $option1 ) = 2;
	my ( $option2 ) = "words";
	my ( $option3 ) = "0.5";
   
	$top2 = $self->Toplevel( -title => "Formal Proximity" );
	my ( $frame1 ) = $top2->Frame()->grid( -row => 0, -column => 0,
			 -ipadx => 10, -ipady => 5, -sticky => 'ew' );
	my ( $frame5 ) = $top2->Frame( -relief=> 'groove', -borderwidth=> 2 )->
	    grid( -row=>3, -column=>0, -ipadx=>10, -ipady=>10, -sticky=>'ew' );
    
	$frame1->Radiobutton( -text => "COLOCATIONS", -value => "COLOCATIONS",
			      -command => sub {
		
				  $entry2->configure( -state => 'disabled' );

				  $button1->configure( -state => 'disabled' );
				  $button2->configure( -state => 'disabled' );
				  $button3->configure( -state => 'disabled' );
			      },
			      -variable=> \$radio )->pack( -side=>'top',
			      -anchor=>'w', -padx=>10, -pady=>10 );

	my ( $frame2 ) = $frame1->Frame()->pack
	    ( -side=>'top', -fill => 'x', -ipadx=> 5, -ipady=>3 );
	$frame2->Label( -text => "Lexical Forms: " )->grid
	    ( -row=>0, -column=> 0 );
	$entry1 = $frame2->Entry( -width => 34,
				  -textvariable => \$pattern1,
				  -selectforeground=> 'white',
				  -selectbackground=> "#000070669645" )
	    ->grid( -row=> 0, -column=> 1, -ipady=> 5 );


	$frame1->Radiobutton( -text=> "SHARED WORDS", -value=> "SHARED WORDS",
			      -command => sub {
				  
				  $entry2->configure( -state => 'normal' );

				  $button1->configure( -state => 'normal' );
				  $button2->configure( -state => 'disabled' );
				  $button3->configure( -state => 'disabled' );
			      },
			      -variable=> \$radio )->pack
				  ( -side=>'top', -anchor=>'w',
				    -padx=>10, -pady=>10 );

	my ( $frame3 ) = $frame1->Frame()->pack
	    ( -side=>'top', -fill => 'x', -ipadx=> 5, -ipady=>3 );
	$frame3->Label( -text => "Partition Key: " )->grid
	    ( -row=>0, -column=>0 );
	$entry2 = $frame3->Entry( -width => 34,
				  -textvariable => \$pattern2,
				  -selectforeground => 'white',
				  -selectbackground => "#000070669645" )
	    ->grid( -row=> 0, -column=> 1 );
	$frame3->Label( -text => "Filter  Words: " )->grid
	    ( -row=>1, -column=> 0 );
	$button1 = $frame3->Optionmenu( -options=> [1, 2, 3, 4, 5, 6, 7,
						    8, 9, 10, 11, 12,
						    13, 14, 15, 16, 17,
						    18, 19, 20],
					-textvariable=> \$option1,
					-variable=> \$option1,
					-width => 30 )->grid
					    ( -row=>1, -column=>1);
	
	$frame1->Radiobutton( -text=>"FORMAL DISTANCES",
			      -value=>"FORMAL DISTANCES",
			      -command => sub {
			      
				  $entry2->configure( -state => 'normal' );

				  $button1->configure( -state => 'normal' );
				  $button2->configure( -state => 'normal' );
				  $button3->configure( -state => 'normal' );
			      },
			      -variable=> \$radio )->pack
				  ( -side=>'top', -anchor=>'w',
				    -padx=>10, -pady=>10 );
	
	my ( $frame4 ) = $frame1->Frame()->pack
	    ( -side=>'top', -fill => 'x', -ipadx=> 5, -ipady=>3 );
	$frame4->Label( -text => "Minimal  Units: " )->grid
	    ( -row=>0, -column=>0 );
	$button2 = $frame4->Optionmenu( -options=> ["words", "bigrams"],
					-textvariable=> \$option2,
					-variable=> \$option2,
					-width => 30 )->grid
					    ( -row=>0, -column=>1);
	$frame4->Label( -text => "Minimal Score: " )->grid
	    ( -row=>1, -column=> 0 );
	$button3 = $frame4->Optionmenu( -options=> ['0.3', '0.4', '0.5',
						    '0.6', '0.7', '0.8',
						    '0.9', '1', '1.1',
						    '1.2', '1.3', '1.4',
						    '1.5', '1.6', '1.7',
						    '1.8', '1.9', '2',
						    '2.5'],
					-textvariable=> \$option3,
					-variable=> \$option3,
					-width => 30 )->grid
					    ( -row=>1, -column=> 1 );

	$frame5->Button( -text => "OK", -command => sub {

	    if ( $lock &&! Exists( $top3 )) {
		my ( $results, $entries );

		$self->Busy();
		$lock = 0;

		unless (( $a[0] ) && ( $a[0] eq "locked" )) {
		    $keys = $self->makeIndex();
		}
		if ( $pattern1 ) {
	    
		    $pattern1 =~ s/^\W+//;
		    @$entries = split( /\W/, $pattern1 );
		    $results = $self->searchColocations( $entries );
		}
	
		unless ( $radio eq 'COLOCATIONS' ) {

		    my ( $status, $key, $value, $size );
		    $key = $value = 0;

		    $status = $FORMS->get( "_size", $size );
		    unless ( $status ) {

			for ( $status = $FORMS->seq( $key, $value, R_FIRST );
			      $status == 0;
			      $status = $FORMS->seq( $key, $value, R_NEXT )) {
			    $FORMS->del( $key );
			}
		    }
		    if ( $results ) {
			$self->filterForms( $results );
		    }
		    if ( $pattern2 ) {
			$pattern2 =~ s/^\W+//g;
			if (! exists( $keys->{$pattern2} )) {
			    $lock = 1;
			    $self->BackTrace( "Invalid key ! ");
			}
		    }
		    $status = $DIST->get( "_size", $size );
		    unless ( $status ) {
			
			for ( $status = $DIST->seq( $key, $value, R_FIRST );
			      $status == 0;
			      $status = $DIST->seq( $key,$value, R_NEXT )) {
			    $DIST->del( $key );
			}
		    }
		    $status = $FORMS->get( "_size", $size );
		    $self->checkDistances( $status? $BTREE: $FORMS );
		    
		    if ( $pattern2 ) {
			$self->sortDistances( $keys, $pattern2 );
		    }
		    if ( $radio eq 'SHARED WORDS' ) {
			$results = $self->SharedWords( $option1 );
		    }
		    else {
			$results = $self->FormalDistances( $option1, $option2,
							   $option3 );
		    }
		}

		$self->showProximity( $results, $entries, $radio );
		$self->Unbusy;
	    }
	    else {
		$self->BackTrace( "Could not access the database. $!\n" );
	    }
	    $lock = 1;

	})->grid( -row => 0, -column => 0, -padx => 40 );
    
	$button_cancel = $frame5->Button( -text => "Cancel",
					  -command => sub {

	if ( $lock ) {

	    $self->configure( -state => 'normal' );
	    $top2->destroy();
	    
	    undef( $BTREE );
	    undef( $FORMS );
	    undef( $DIST );
	    
	    untie( @a );
	    untie( %b );
	    untie( %f );
	    untie( %d );
	    
	    $self->tellStatusbar( "Proximity session closed." );
	}
	else { $self->BackTrace( "Could not unlock the database. $!\n" ); }

	})->grid( -row => 0, -column => 2, -padx => 40 );

	@ranges = $self->tagRanges( 'sel' );
	if ( @ranges ) {
	
	    my ( $first_line, $first_colon, $last_line, $last_colon );
	    
	    $first = shift( @ranges );
	    $last = shift( @ranges );
	    ( $first_line, $first_colon ) = split( /\./, $first );
	    ( $last_line, $last_colon ) = split( /\./, $last );
	    
	    unless( $first_line == $last_line ) { $last = $first. ' lineend'; }
	    $entry1->insert( 'end', $self->get( $first , $last ));
	    $entry1->selectionTo( 'end' );
	    $entry1->focus();
	}
	else {
	    
	    eval {
		$selection = $self->SelectionGet( -selection => "PRIMARY");
	    };
	    unless( $@ ) {
		if ( defined( $selection ) and length( $selection )) {
		    $entry1->insert( 'end', $selection );
		    $entry1->selectionTo( 'end' );
		    $entry1->focus();
		}
	    }
	}
	
	$entry2->configure( -state => 'disabled' );
	$button1->configure( -state => 'disabled' );
	$button2->configure( -state => 'disabled' );
	$button3->configure( -state => 'disabled' );

	$top2->protocol( 'WM_DELETE_WINDOW'=> sub { $button_cancel->invoke; });
	$top2->resizable( 0, 0 );

	return( $self->tellStatusbar( "Proximity session opened." ));
    }
    return( 0 );
}


1;

__END__


=head1 NAME
    
TextTools - perl extensions library for "edxml"
    
=head1 SYNOPSIS
    
use Tk::TextTools;

$buffer= $toplevel->Scrolled( 'TextTools' );
$buffer->loadStatusbar( $statusbar );
$menu  = $buffer->menu();

=head1 DESCRIPTION

This module provides routines for text manipulation,
such as search and replacement tools, and G.U.I. configuration.
It overrides the module "TextEdit.pm" coming with perl_Tk.

=head1 AUTHOR
    
Calin MOSUT, "Babes-Bolyai" University
Faculty of Letters, A.M.L. Departement
Bulevard Horea, 31, 3400 Cluj-Napoca
Romania

Phone : (40/)64 432 540
Email : cmosut@personal.ro

=cut
