#!/usr/local/bin/perl5

use HTMLQC;
use SHTML;
use File::Basename;
use Cwd;
use Getopt::Long;
use File::Find;
my ($webroot,$single,$fileroot,$confine,$gethelp)
    = ('.',undef,undef,undef,undef);
unless(GetOptions("r=s" => \$webroot,
                  "s"   => \$single,
                  "d=s" => \$fileroot,
                  "c"   => \$confine,
                  "h"   => \$gethelp
                  ))
{
    warn "Bad options\n";
    usage_summary();
}

if ($gethelp)
{
    usage_summary();
    exit;
}

chdir($fileroot);

my (@urls,%procurls,%fullurl);
(push @urls,$ARGV[0]) || die "No URL specified";
$procurls{$ARGV[0]} = "root";

while (defined(my $url = pop @urls))
{
    unless(open(D,"<$fileroot/$url"))
    {
        print "Couldn't open ($url) <- ($procurls{$url}): $!\n";
        next;
    }

    next if ($url =~ /gif|jpg|jpeg|pdf$/i);
    next if ($confine and $url !~ m#$webroot/#);
    { local $/; $_ = <D>; }
    close(D);
    $_ = striptohtml($_);
    extract_urls($_,$url) unless($single);
    unless(htmlqc($_))
    {
        print "In file $url:\n$HTMLQC::error\n";
    }
}

unless ($single)
{
    print "\n\nFiles in directory ($webroot) not in HTML\n\n";
    chdir($fileroot);
    find(\&vrfyfile,"$webroot");

    print "\n\nFiles/Images/References found in HTML\n\n";

    for my $url (keys %fullurl)
    {
        print "$url\n";
    }
}

sub vrfyfile
{
    return if (-d "$fileroot/$File::Find::name");
    my $file = $File::Find::name;
    $file =~ s#^#/# unless ($file =~ m#^/#);
    print "$file is not referenced in HTML\n"
        unless(defined($fullurl{$file}));
}

sub extract_urls
{
    my ($file,$topurl) = @_;

    foreach $_ (split "\n",$file)
    {
        if (m/<a.*href=(.*)[> ].*/i or m/<img.*src=(.*)[> ].*/i)
        {
            my $url = $1;
            $url =~ s/\"//g;
            $url =~ s/>.*//g;
            $url =~ s/^\s+//;
            $url =~ s/\ .*//g;
            next if ($url =~ /cgi-bin/i);
            next if ($url =~ /\.map/i);
            next if ($url =~ /\.zip/i);
            next if ($url =~ /^\S+:/i);
            my ($protocol,$machine,$path) 
                = (decipher_url($url))[0,3,5];
            next if (defined($protocol));

            $path = $url unless $path;
            next if ($path =~ /\#.*/);
            print "Line $_ looks bad\n" if (length($path) <1);
            $path .= "/index.html" if ($path =~ /.*\/$/);
            unless ($path =~ m#^/#)
            {
                $path = dirname($topurl) ."/". $path;
            }
            $path = expand_path($path);
            $path =~ s#//#/#g;
            push(@urls,$path) unless $procurls{$path};
            $fullurl{$path} = 1 unless $procurls{$path};
            $procurls{$path} = $topurl;
        }
    }
}
    
sub usage_summary()
{
    print <<EOF
Usage: chklinks [-s] [-r root] [-d fileroot] [-c] [-h] htmlfile

Extracts links from HTML (<IMG> and <A> tags only) and checks 
for destination links and HTML validity.

Where:

s  -  Check a single page for link and HTML validity

r  -  Specify the root directory for the website
      eg. To check all files beneath http://www.foo.com/bar 
          you would set the root to bar

d  -  Specify the physical directory for the website
      eg. /usr/local/etc/httpd/service

c  -  Confine checks to within the root directory
      eg. Using: chklinks.pl -r bar -c 
          /bar/index.html would be expanded
          /index.html wouldn't 

h  -  Show this help

EOF
    ;
}
