#!/usr/local/bin/perl -w

use LWP::Simple;
use Getopt::Std;

$progname = $0;
$progname =~ s,.*/,,;
$progname =~ s/\.\w*$//;

$VERSION = sprintf("%d.%02d", 
                   q$Revision: 1.18 $ =~ /(\d+)\.(\d+)/);

$opt_h = undef;  
$opt_v = undef;  
$opt_t = undef;  

unless (getopts("hvt:")) {
    usage();
}

if ($opt_v) {
    require LWP;
    my $DISTNAME = 'libwww-perl-' . LWP::Version();
    die <<"EOT";
This is lwp-mirror version $VERSION ($DISTNAME)

Copyright 1995-1996, Gisle Aas.

This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.
EOT
}

$url  = shift or usage();
$file = shift or usage();
usage() if $opt_h or @ARGV;

if (defined $opt_t) {
    $opt_t =~ /^(\d+)([smh])?/;
    die "$progname: Illegal timeout value!\n" unless defined $1;
    $timeout = $1;
    $timeout *= 60   if ($2 eq "m");
    $timeout *= 3600 if ($2 eq "h");
    $LWP::Simple::ua->timeout($timeout);
}

$rc = mirror($url, $file);

if ($rc == 304) {
    print STDERR "$progname: $file is up to date\n"
} elsif (!is_success($rc)) {
    print STDERR ("$progname: $rc ", 
                  status_message($rc), 
                  "   ($url)\n");
    exit 1;
}
exit;


sub usage
{
    die <<"EOT";
Usage: $progname [-options] <url> <file>
    -v           print version number of program
    -t <timeout> Set timeout value
EOT
}
